from __future__ import unicode_literals

import json
import base64

from mock import patch

from django.core.urlresolvers import reverse_lazy
from django.test import TestCase

from core.models import ElectionResults


class TestResultsStatusView(TestCase):
    """ """
    def setUp(self):
        """Setup all needed endpoints."""
        self.election_id = 5
        req_kwargs = {'election_id': self.election_id}
        self.url = reverse_lazy(
            'api:status',
            kwargs=req_kwargs
        )

    def test_election_not_found(self):
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 404)

    def test_election_status(self):
        results = ElectionResults(election_id=self.election_id)
        results.save()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        json_response = json.loads(response.content)
        status = json_response.pop("result_status_choices")
        self.assertEqual(
            status,
            ElectionResults.ResultStatus.EXTRACTION
        )


class TestResultsCleanView(TestCase):
    """ """
    def setUp(self):
        """Setup all needed endpoints."""
        self.election_id = 5
        req_kwargs = {'election_id': self.election_id}
        self.url = reverse_lazy(
            'api:clean',
            kwargs=req_kwargs
        )

    def test_results_not_found(self):
        response = self.client.delete(self.url)
        self.assertEqual(response.status_code, 404)

    def test_clean_ok(self):
        results = ElectionResults(election_id=self.election_id)
        results.save()
        response = self.client.delete(self.url)
        self.assertEqual(response.status_code, 204)


class TestResultsStartView(TestCase):
    """ """
    def setUp(self):
        """Setup all needed endpoints."""
        self.election_id = 5
        req_kwargs = {'election_id': self.election_id}
        self.url = reverse_lazy(
            'api:generate',
            kwargs=req_kwargs
        )
        # Post data to generate the results
        self.dummy_root_b64 = base64.b64encode("dummy_root")
        dummy_b64 = base64.b64encode("dummy")
        self.postdata = {
            "cs_pvt_key": dummy_b64,
            "elec_proc_id": 1,
            "sge_cert": dummy_b64,
            "election_pvt_key": dummy_b64,
            "vs_cert": dummy_b64,
            "root_cert": self.dummy_root_b64,
            "secret_list": ["secret1", "secret2"]
        }

    # Helper function to mock a function twice with different behaviour
    def certificate_side_effect(self, *args, **kwargs):
        if args[0] == "dummy_root":
            return (1, None)
        else:
            return (None, "dummy")

    # Unit tests
    def test_no_postdata(self):
        response = self.client.post(self.url)
        self.assertEqual(response.status_code, 400)

    def test_error_response(self):
        results = ElectionResults(election_id=self.election_id)
        results.save()
        response = self.client.post(self.url, self.postdata)
        self.assertEqual(response.status_code, 400)
        json_response = json.loads(response.content)
        self.assertTrue(json_response.pop("error"))

    @patch('core.views_api.is_worker_running')
    def test_worker_running(self, mock_fn):
        mock_fn.return_value = True
        results = ElectionResults(election_id=self.election_id)
        results.save()
        response = self.client.post(self.url, self.postdata)
        self.assertEqual(response.status_code, 400)
        json_response = json.loads(response.content)
        self.assertTrue(json_response.pop("error"))

    def test_published_response(self):
        # Set dummy results
        results = ElectionResults(
            election_id=self.election_id,
            result_status_choices=ElectionResults.ResultStatus.PUBLISHED,
            election_results=json.dumps(self.postdata)
        )
        results.save()
        response = self.client.post(self.url, self.postdata)
        self.assertEqual(response.status_code, 200)
        json_response = json.loads(response.content)
        self.assertFalse(json_response.pop("error"))
        self.assertEqual(json_response.pop("message"), self.postdata)

    def test_invalid_vs_cert(self):
        response = self.client.post(self.url, self.postdata)
        json_response = json.loads(response.content)
        self.assertEqual(response.status_code, 400)
        self.assertTrue(json_response.pop("error"))

    @patch('core.views_api.pkiutils.getPublicKeyFromCertificate')
    def test_invalid_root_cert(self, mock_fn):
        mock_fn.side_effect = self.certificate_side_effect
        response = self.client.post(self.url, self.postdata)
        json_response = json.loads(response.content)
        self.assertEqual(response.status_code, 400)
        self.assertTrue(json_response.pop("error"))

    @patch('core.views_api.pkiutils.verifyCertificateChain')
    @patch('core.views_api.pkiutils.getPublicKeyFromCertificate')
    def test_invalid_chain_cert(self, mock_getpubkey, mock_verifychain):
        mock_getpubkey.return_value = (None, "dummy_cert")
        mock_verifychain.return_value = False
        response = self.client.post(self.url, self.postdata)
        json_response = json.loads(response.content)
        self.assertEqual(response.status_code, 400)
        self.assertTrue(json_response.pop("error"))

    @patch('core.views_api.pkiutils.verifyCertificateChain')
    @patch('core.views_api.pkiutils.getPublicKeyFromCertificate')
    def test_ok_response(self, mock_getpubkey, mock_verifychain):
        mock_getpubkey.return_value = (None, "dummy_cert")
        mock_verifychain.return_value = True
        response = self.client.post(self.url, self.postdata)
        json_response = json.loads(response.content)
        self.assertEqual(response.status_code, 200)
        self.assertFalse(json_response.pop("error"))


class TestPingView(TestCase):
    """ """
    def setUp(self):
        """Setup all needed endpoints."""
        self.url = reverse_lazy('api:ping')

    def test_server_alive(self):
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
